aboutsummaryrefslogtreecommitdiff
path: root/builtin-stripspace.c
diff options
context:
space:
mode:
authorCarlos Rica <jasampler@gmail.com>2007-07-11 20:50:34 +0200
committerJunio C Hamano <gitster@pobox.com>2007-07-11 21:17:07 -0700
commit975e0daf5edba09060147d643e628b8c89e1d467 (patch)
treea68a6f1aba6261f2d6aa020600b994deb5223753 /builtin-stripspace.c
parent73f893605050c291dd8c4d8f3f50b8fec47dcf51 (diff)
downloadgit-975e0daf5edba09060147d643e628b8c89e1d467.tar.gz
git-975e0daf5edba09060147d643e628b8c89e1d467.tar.xz
Function stripspace now gets a buffer instead file descriptors.
An implementation easier to call from builtins. It is designed to be used from the upcoming builtin-tag.c and builtin-commit.c, because both need to remove unwanted spaces from messages. Signed-off-by: Carlos Rica <jasampler@gmail.com> Signed-off-by: Junio C Hamano <gitster@pobox.com>
Diffstat (limited to 'builtin-stripspace.c')
-rw-r--r--builtin-stripspace.c61
1 files changed, 37 insertions, 24 deletions
diff --git a/builtin-stripspace.c b/builtin-stripspace.c
index d8358e28f..0c970aa94 100644
--- a/builtin-stripspace.c
+++ b/builtin-stripspace.c
@@ -2,12 +2,11 @@
#include "cache.h"
/*
* Remove trailing spaces from a line.
*
* If the line ends with newline, it will be removed too.
- * Returns the new length of the string.
*/
-static int cleanup(char *line, int len)
+static size_t cleanup(char *line, size_t len)
{
if (len) {
if (line[len - 1] == '\n')
@@ -19,7 +18,6 @@ static int cleanup(char *line, int len)
break;
len--;
}
- line[len] = 0;
}
return len;
}
@@ -28,52 +26,67 @@ static int cleanup(char *line, int len)
* Remove empty lines from the beginning and end
* and also trailing spaces from every line.
*
+ * Note that the buffer will not be NUL-terminated.
+ *
* Turn multiple consecutive empty lines between paragraphs
* into just one empty line.
*
* If the input has only empty lines and spaces,
* no output will be produced.
*
+ * If last line has a newline at the end, it will be removed.
+ *
* Enable skip_comments to skip every line starting with "#".
*/
-void stripspace(FILE *in, FILE *out, int skip_comments)
+size_t stripspace(char *buffer, size_t length, int skip_comments)
{
int empties = -1;
- int alloc = 1024;
- char *line = xmalloc(alloc);
+ size_t i, j, len, newlen;
+ char *eol;
- while (fgets(line, alloc, in)) {
- int len = strlen(line);
+ for (i = j = 0; i < length; i += len, j += newlen) {
+ eol = memchr(buffer + i, '\n', length - i);
+ len = eol ? eol - (buffer + i) + 1 : length - i;
- while (len == alloc - 1 && line[len - 1] != '\n') {
- alloc = alloc_nr(alloc);
- line = xrealloc(line, alloc);
- fgets(line + len, alloc - len, in);
- len += strlen(line + len);
- }
-
- if (skip_comments && line[0] == '#')
+ if (skip_comments && len && buffer[i] == '#') {
+ newlen = 0;
continue;
- len = cleanup(line, len);
+ }
+ newlen = cleanup(buffer + i, len);
/* Not just an empty line? */
- if (len) {
+ if (newlen) {
+ if (empties != -1)
+ buffer[j++] = '\n';
if (empties > 0)
- fputc('\n', out);
+ buffer[j++] = '\n';
empties = 0;
- fputs(line, out);
- fputc('\n', out);
+ memmove(buffer + j, buffer + i, newlen);
continue;
}
if (empties < 0)
continue;
empties++;
}
- free(line);
+
+ return j;
}
int cmd_stripspace(int argc, const char **argv, const char *prefix)
{
- stripspace(stdin, stdout, 0);
+ char *buffer;
+ unsigned long size;
+
+ size = 1024;
+ buffer = xmalloc(size);
+ if (read_pipe(0, &buffer, &size))
+ die("could not read the input");
+
+ size = stripspace(buffer, size, 0);
+ write_or_die(1, buffer, size);
+ if (size)
+ putc('\n', stdout);
+
+ free(buffer);
return 0;
}
@@ -2,12 +2,11 @@
#include "cache.h"
/*
- * Remove trailing spaces from a line.
+ * Returns the length of a line, without trailing spaces.
*
* If the line ends with newline, it will be removed too.